import { Request, Response } from 'express';
import { prisma, __path, log } from '../../../index.js';
import { parseIntOrUndefined, parseDynamicSortBy, parseIntRelation } from '../../../assets/helper.js';

// Get storageLocation.
async function get(req: Request, res: Response) {
	if (req.query.sort === undefined) {
		req.query.sort = 'id';
	}
	if (req.query.order === undefined) {
		req.query.order = 'asc';
	}
	if (req.query.search === undefined) {
		req.query.search = '';
	}

	if (req.query.id) {
		prisma.storageLocation
			.findUnique({
				where: {
					id: parseInt(req.query.id.toString())
				},
				// Get storageUnit from relation.
				include: {
					storageUnit: true
				}
			})
			.then((items) => {
				if (items) {
					res.status(200).json(items);
				} else {
					res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageLocation does not exist' });
				}
			})
			.catch((err) => {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			});
	} else {
		// Get all items
		const itemCountNotFiltered = await prisma.storageLocation.count({});

		// Get all items (filtered)
		const itemCountFiltered = await prisma.storageLocation.count({
			where: {
				name: {
					// @ts-ignore
					contains: req.query.search.length > 0 ? req.query.search : ''
				}
			},
			orderBy: parseDynamicSortBy(req.query.sort.toString(), req.query.order.toString())
		});

		prisma.storageLocation
			.findMany({
				take: parseIntOrUndefined(req.query.limit),
				skip: parseIntOrUndefined(req.query.offset),
				where: {
					name: {
						// @ts-ignore
						contains: req.query.search.length > 0 ? req.query.search : ''
					}
				},
				// Get storageUnit from relation.
				include: {
					storageUnit: true
				},
				orderBy: parseDynamicSortBy(req.query.sort.toString(), req.query.order.toString()),
			})
			.then((items) => {
				if (items) {
					res.status(200).json({ total: itemCountFiltered, totalNotFiltered: itemCountNotFiltered, items: items });
				} else {
					res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageLocation does not exist' });
				}
			})
			.catch((err) => {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			});
	}
}

// Create storageLocation.
function post(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.name) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}
	// Create storageLocation with existing storageUnit.
	prisma.storageLocation
		.create({
			data: {
				name: req.body.name,
				storageUnitId: parseInt(req.body.storageUnitId) || undefined
			},
			select: {
				id: true
			}
		})
		.then((data) => {
			res.status(201).json({ status: 'CREATED', message: 'Successfully created storageLocation', id: data.id });
		})
		.catch((err) => {
			// Check if an entry already exists.
			if (err.code === 'P2002') {
				// P2002 -> "Unique constraint failed on the {constraint}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'storageLocation already exists' });
			} else if (err.code == 'P2003') {
				// P2003 -> "Foreign key constraint failed on the field: {field_name}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				// FIXME: Is this errormessage right?
				res.status(404).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageUnitId does not exist' });
			} else if (err.code == 'P2000') {
				// P2000 -> "The provided value for the column is too long for the column's type. Column: {column_name}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(404).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more fields exceed the maximum length restriction' });
			} else {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			}
		});
}

// Update storageLocation. -> Only existing contactInfo.
async function patch(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id || !req.body.name || !req.body.storageUnitId) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Check if the storageLocation id exists. If not return 410 Gone.
	try {
		const result = await prisma.storageLocation.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(404).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageLocation does not exist' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	prisma.storageLocation
		.update({
			where: {
				id: parseInt(req.body.id)
			},
			data: {
				name: req.body.name,
				storageUnit: parseIntRelation(req.body.storageUnitId)
			},
			select: {
				id: true
			}
		})
		.then((data) => {
			res.status(201).json({ status: 'UPDATED', message: 'Successfully updated storageLocation', id: data.id });
		})
		.catch((err) => {
			// Check if an entry already exists.
			if (err.code === 'P2002') {
				// P2002 -> "Unique constraint failed on the {constraint}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'storageLocation already exists' });
			} else if (err.code == 'P2003') {
				// P2003 -> "Foreign key constraint failed on the field: {field_name}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				// FIXME: Is this errormessage right?
				res.status(404).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageUnitId does not exist' });
			} else if (err.code == 'P2000') {
				// P2000 -> "The provided value for the column is too long for the column's type. Column: {column_name}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(404).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more fields exceed the maximum length restriction' });
			} else {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			}
		});
}

// Delete storageLocation.
async function del(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Does the id exist? If not return 410 Gone.
	try {
		const result = await prisma.storageLocation.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageLocation does not exist' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	prisma.storageLocation
		.delete({
			where: {
				id: parseInt(req.body.id)
			}
		})
		.then(() => {
			res.status(200).json({ status: 'DELETED', message: 'Successfully deleted storageLocation' });
		})
		.catch((err) => {
			log.db.error(err);
			res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
		});
}

export default { get, post, patch, del };
