import { Request, Response } from 'express';
import { prisma, __path, log } from '../../../index.js';
import { contactType } from '@prisma/client';
import { parseDynamicSortBy, parseIntOrUndefined } from '../../../assets/helper.js';

// Get storageUnit.
async function get(req: Request, res: Response) {
	if (req.query.sort === undefined) {
		req.query.sort = 'id';
	}
	if (req.query.order === undefined) {
		req.query.order = 'asc';
	}
	if (req.query.search === undefined) {
		req.query.search = '';
	}
	if (req.query.id) {
		prisma.storageUnit
			.findUnique({
				where: {
					id: parseInt(req.query.id.toString())
				},
				// Get contactInfo and StorageLocation from relation.
				include: {
					contactInfo: true,
					StorageLocation: true
				}
			})
			.then((items) => {
				if (items) {
					res.status(200).json(items);
				} else {
					res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageUnit does not exist' });
				}
			})
			.catch((err) => {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			});
	} else {
		// Get all items
		const itemCountNotFiltered = await prisma.storageUnit.count({});

		// Get all items (filtered)
		const itemCountFiltered = await prisma.storageUnit.count({
			where: {
				name: {
					// @ts-ignore
					contains: req.query.search.length > 0 ? req.query.search : ''
				}
			},
			orderBy: parseDynamicSortBy(req.query.sort.toString(), req.query.order.toString())
		});

		prisma.storageUnit
			.findMany({
				take: parseIntOrUndefined(req.query.limit),
				skip: parseIntOrUndefined(req.query.offset),
				// Get contactInfo and StorageLocation from relation.
				include: {
					contactInfo: true,
					StorageLocation: true
				},
				where: {
					name: {
						// @ts-ignore
						contains: req.query.search.length > 0 ? req.query.search : ''
					}
				},
				orderBy: parseDynamicSortBy(req.query.sort.toString(), req.query.order.toString())
			})
			.then((items) => {
				if (items) {
					res.status(200).json({ total: itemCountFiltered, totalNotFiltered: itemCountNotFiltered, items: items });
				} else {
					res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageUnit does not exist' });
				}
			})
			.catch((err) => {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			});
	}
}

// Create storageUnit.
function post(req: Request, res: Response) {
	// If the frontend wants to create a StorageUnit with non-existing ContactInfo.
	if (req.body.locationId === 'META_CREATENEW') {
		// Check if required fields are present.
		if (!req.body.street || !req.body.houseNumber || !req.body.zipCode || !req.body.city || !req.body.country || !req.body.name) {
			res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
			return;
		}

		// Create storageUnit and location.
		prisma.storageUnit
			.create({
				data: {
					name: req.body.name,
					contactInfo: {
						create: {
							type: contactType.storageUnit,
							street: req.body.street,
							houseNumber: req.body.houseNumber,
							zipCode: req.body.zipCode,
							city: req.body.city,
							country: req.body.country
						}
					}
				},
				select: {
					id: true
				}
			})
			.then((data) => {
				res.status(201).json({ status: 'CREATED', message: 'Successfully created storageUnit', id: data.id });
			})
			.catch((err) => {
				// Check if an entry already exists.
				if (err.code === 'P2002') {
					// P2002 -> "Unique constraint failed on the {constraint}"
					// https://www.prisma.io/docs/reference/api-reference/error-reference
					res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'storageUnit already exists' });
				} else if (err.code == 'P2000') {
					// P2000 -> "The provided value for the column is too long for the column's type. Column: {column_name}"
					// https://www.prisma.io/docs/reference/api-reference/error-reference
					res.status(404).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more fields exceed the maximum length restriction' });
				} else {
					log.db.error(err);
					res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
				}
			});
	} else {
		// Check if required fields are present.
		if (!req.body.name || !req.body.locationId) {
			res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
			return;
		}
		// Create storageUnit with existing location.
		prisma.storageUnit
			.create({
				data: {
					name: req.body.name,
					contactInfo: {
						connect: {
							id: parseInt(req.body.locationId)
						}
					}
				},
				select: {
					id: true
				}
			})
			.then((data) => {
				res.status(201).json({ status: 'CREATED', message: 'Successfully created storageUnit', id: data.id });
			})
			.catch((err) => {
				// Check if an entry already exists.
				if (err.code === 'P2002') {
					// P2002 -> "Unique constraint failed on the {constraint}"
					// https://www.prisma.io/docs/reference/api-reference/error-reference
					res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'storageUnit already exists' });
				} else if (err.code == 'P2000') {
					// P2000 -> "The provided value for the column is too long for the column's type. Column: {column_name}"
					// https://www.prisma.io/docs/reference/api-reference/error-reference
					res.status(404).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more fields exceed the maximum length restriction' });
				} else {
					log.db.error(err);
					res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
				}
			});
	}
}

// Update storageUnit. -> Only existing contactInfo.
async function patch(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id || !req.body.name || !req.body.locationId) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Check if the storageUnit id exists. If not return 410 Gone.
	try {
		const result = await prisma.storageUnit.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageUnit does not exist' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	// Check if the locationId(contactInfo) exists. If not return 410 Gone.
	try {
		const result = await prisma.contactInfo.findUnique({
			where: {
				id: parseInt(req.body.locationId)
			}
		});

		if (result === null) {
			res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'locationId does not exist' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	prisma.storageUnit
		.update({
			where: {
				id: parseInt(req.body.id)
			},
			data: {
				name: req.body.name,
				contactInfo: {
					connect: {
						id: parseInt(req.body.locationId) // TODO: Rename to contactInfoId
					}
				}
			},
			select: {
				id: true
			}
		})
		.then((data) => {
			res.status(201).json({ status: 'UPDATED', message: 'Successfully updated storageUnit', id: data.id });
		})
		.catch((err) => {
			// Check if an entry already exists.
			if (err.code === 'P2002') {
				// P2002 -> "Unique constraint failed on the {constraint}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'storageUnit already exists' });
			} else if (err.code == 'P2000') {
				// P2000 -> "The provided value for the column is too long for the column's type. Column: {column_name}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(404).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more fields exceed the maximum length restriction' });
			} else {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			}
		});
}

// Delete storageUnit.
async function del(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Does the id exist? If not return 410 Gone.
	try {
		const result = await prisma.storageUnit.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'storageUnit does not exist' });

			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	prisma.storageUnit
		.delete({
			where: {
				id: parseInt(req.body.id)
			}
		})
		.then(() => {
			res.status(200).json({ status: 'DELETED', message: 'Successfully deleted storageUnit' });
		})
		.catch((err) => {
			log.db.error(err);
			res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
		});
}

export default { get, post, patch, del };
